@@ -1,3 +1,21 @@
+/* GDM - The GNOME Display Manager
+ * Copyright (C) 1998, 1999, 2000 Martin K. Petersen <mkp@mkp.net>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+
 #include "config.h"
 
 #include <gtk/gtk.h>
@@ -71,13 +89,15 @@ set_text (GreeterItemInfo *info, const char *text)
 					      info->item);
 }
 
-static void
-user_pw_activate (GtkEntry *entry, GreeterItemInfo *info)
+void
+greeter_item_pam_login (GtkEntry *entry, GreeterItemInfo *info)
 {
   const char *str;
   char *tmp;
   GreeterItemInfo *error_info;
 
+  greeter_ignore_buttons (TRUE);
+
   str = gtk_entry_get_text (GTK_ENTRY (entry));
   if (greeter_probably_login_prompt &&
       /* evilness */
@@ -124,11 +144,17 @@ key_press_event (GtkWidget *entry, GdkEventKey *event, gpointer data)
        event->keyval == GDK_KP_Tab) &&
       (event->state & (GDK_CONTROL_MASK|GDK_MOD1_MASK|GDK_SHIFT_MASK)) == 0)
     {
-      g_signal_emit_by_name (entry,
-		             "insert_at_cursor",
-		             "\t");
-      return TRUE;
-  }
+	GreeterItemInfo *entry_info = greeter_lookup_id ("user-pw-entry");
+	if (entry_info && entry_info->item &&
+	    GNOME_IS_CANVAS_WIDGET (entry_info->item) &&
+	    GTK_IS_ENTRY (GNOME_CANVAS_WIDGET (entry_info->item)->widget))
+	{
+		GtkWidget *entry;
+		entry = GNOME_CANVAS_WIDGET (entry_info->item)->widget;
+		greeter_item_pam_login (GTK_ENTRY (entry), entry_info);
+		return TRUE;
+        }
+    }
 
   return FALSE;
 }
@@ -160,7 +186,7 @@ greeter_item_pam_setup (void)
 	}
 
       g_signal_connect (entry, "activate",
-			G_CALLBACK (user_pw_activate), entry_info);
+			G_CALLBACK (greeter_item_pam_login), entry_info);
       g_signal_connect (G_OBJECT (entry), "key_press_event",
 		        G_CALLBACK (key_press_event), NULL);
     }
@@ -218,7 +244,9 @@ greeter_item_pam_message (const char *message)
       /* HAAAAAAACK.  Sometimes pam sends many many messages, SO
        * we try to collect them until the next prompt or reset or
        * whatnot */
-      if (!replace_msg)
+      if ( ! replace_msg &&
+	   /* empty message is for clearing */
+	   ! ve_string_empty (message))
 	{
 	  g_object_get (G_OBJECT (message_info->item),
 			"text", &oldtext,
